local key = ModPath .. '	' .. RequiredScript
if _G[key] then return else _G[key] = true end

_G.ABI_Signals = _G.ABI_Signals or {}
ABI_Signals._path = ModPath
ABI_Signals._data_path = SavePath .. 'abi_signals.txt'
ABI_Signals._data = {}
ABI_Signals.settings = {}
ABI_Signals.control = {
	effect_duration_timer = 0
}

function ABI_Signals:reset()
	-- indicator settings
	self.settings.indicator_enabled = true
	self.settings.indicator_color = 1
	self.settings.indicator_texture_path = "assets/guis/textures/armor_break_invulnerability_indicator_"
	-- effect settings
	self.settings.effect_enabled = true
	self.settings.effect_color = "C71585"
	self.settings.effect_texture_path = "assets/guis/textures/armor_break_invulnerability_effect"
end

function ABI_Signals:store_color_settings()
	self.settings.effect_color = string.format('%02x%02x%02x',
		math.floor(self._data.ER * 100), math.floor(self._data.EG * 100), math.floor(self._data.EB * 100)
	)
end

function ABI_Signals:hex_color_rgb(str_hex_color) 
	return {
		tonumber(string.sub(str_hex_color, 1, 2), 16) / 100,
		tonumber(string.sub(str_hex_color, 3, 4), 16) / 100,
		tonumber(string.sub(str_hex_color, 5, 6), 16) / 100
	}
end

function ABI_Signals:settings_to_data()
	-- indicator settings -> data
	self._data.indicator_enabled = self.settings.indicator_enabled
	self._data.indicator_color = self.settings.indicator_color
	-- effect settings -> data
	self._data.effect_enabled = self.settings.effect_enabled
	local effect_rgb = self:hex_color_rgb(self.settings.effect_color)
	self._data.ER = effect_rgb[1]
	self._data.EG = effect_rgb[2]
	self._data.EB = effect_rgb[3]
end

function ABI_Signals:load_custom_textures()
	for _, file in pairs(file.GetFiles(ABI_Signals._path.. "assets/guis/textures/")) do
		DB:create_entry(Idstring("texture"), Idstring("assets/guis/textures/".. file:gsub(".texture", "")), ABI_Signals._path.. "assets/guis/textures/".. file)
	end
end

function ABI_Signals:load()
	self:reset()
	local file = io.open(self._data_path, 'r')
	if file then
		for k, v in pairs(json.decode(file:read('*all')) or {}) do
			self.settings[k] = v
		end
		file:close()
	end
	self:settings_to_data()
	self:load_custom_textures()
end

function ABI_Signals:save()
	local file = io.open(self._data_path, 'w+')
	if file then
		self:store_color_settings()
		file:write(json.encode(self.settings))
		file:close()
	end
end

-- INI: just for developers and curious
function ABI_Signals:DebugEnabled() 
	return false
end

function ABI_Signals:log(a,...)
	if not self:DebugEnabled() then 
		return
	end
	if Console then
		return Console:log(a,...)
	else
		return log("[ABI_Signals] " .. tostring(a))
	end
end
-- END: just for developers and curious

Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_ABI_Signals", function(loc)
	if file.DirectoryExists(ABI_Signals._path .. "loc/") then
		for _, filename in pairs(file.GetFiles(ABI_Signals._path .. "loc/")) do
			local str = filename:match('^(.*).json$')
			if str and Idstring(str) and Idstring(str):key() == SystemInfo:language():key() then
				loc:load_localization_file(ABI_Signals._path .. "loc/" .. filename)
				break
			end
		end
	end
	loc:load_localization_file(ABI_Signals._path .. "loc/english.json", false)
end)

Hooks:Add('MenuManagerInitialize', 'MenuManagerInitialize_ABI_Signals', function(menu_manager)
	-- INDICATOR TOOGLE
	function MenuCallbackHandler:ABI_SignalsSetIndicator(item)
		ABI_Signals.settings.indicator_enabled = item:value() == 'on' and true or false
	end
	-- INDICATOR COLOR MULTI-CHOICE
	function MenuCallbackHandler:ABI_SignalsSetIndicatorColor(item)
		ABI_Signals.settings.indicator_color = item:value()
	end

	-- EFFECT TOOGLE
	function MenuCallbackHandler:ABI_SignalsSetEffect(item)
		ABI_Signals.settings.effect_enabled = item:value() == 'on' and true or false
	end
	-- EFFECT COLOR SLIDERS
	function MenuCallbackHandler:ABI_SignalsSetRedEffect(item)
		ABI_Signals._data.ER = tonumber(item:value())
		ABI_Signals:store_color_settings()
	end

	function MenuCallbackHandler:ABI_SignalsSetGreenEffect(item)
		ABI_Signals._data.EG = tonumber(item:value())
		ABI_Signals:store_color_settings()
	end

	function MenuCallbackHandler:ABI_SignalsSetBlueEffect(item)
		ABI_Signals._data.EB = tonumber(item:value())
		ABI_Signals:store_color_settings()
	end

	function MenuCallbackHandler:ABI_SignalsReset(item)
		ABI_Signals:reset()
		ABI_Signals:settings_to_data()
		MenuHelper:ResetItemsToDefaultValue(item, {['abi_signals_toggle_indicator'] = true}, ABI_Signals._data.indicator_enabled)
		MenuHelper:ResetItemsToDefaultValue(item, {['abi_signals_multi_indicator_colour'] = true}, ABI_Signals._data.indicator_color)
		MenuHelper:ResetItemsToDefaultValue(item, {['abi_signals_toggle_effect'] = true}, ABI_Signals._data.effect_enabled)
		MenuHelper:ResetItemsToDefaultValue(item, {['abi_signals_slider_colour_effect_r'] = true}, ABI_Signals._data.ER)
		MenuHelper:ResetItemsToDefaultValue(item, {['abi_signals_slider_colour_effect_g'] = true}, ABI_Signals._data.EG)
		MenuHelper:ResetItemsToDefaultValue(item, {['abi_signals_slider_colour_effect_b'] = true}, ABI_Signals._data.EB)
		ABI_Signals:save()
	end

	function MenuCallbackHandler:ABI_SignalsSave()
		ABI_Signals:save()
	end

	ABI_Signals:load()
	MenuHelper:LoadFromJsonFile(ABI_Signals._path .. 'menu/options.txt', ABI_Signals, ABI_Signals._data)
end)
